@@ -16,7 +16,11 @@
 
 import os
 
+from testtools.matchers import Equals
+
+import snapcraft
 import integration_tests
+from snapcraft.tests.matchers import HasArchitecture
 
 
 class AutotoolsPluginTestCase(integration_tests.TestCase):
@@ -26,4 +30,14 @@ class AutotoolsPluginTestCase(integration_tests.TestCase):
 
         binary_output = self.get_output_ignoring_non_zero_exit(
             os.path.join(self.stage_dir, 'bin', 'test'))
-        self.assertEqual('Hello world\n', binary_output)
+        self.assertThat(binary_output, Equals('Hello world\n'))
+
+    def test_cross_compiling(self):
+        if snapcraft.ProjectOptions().deb_arch != 'amd64':
+            self.skipTest('The test only handles amd64 to arm64')
+
+        self.run_snapcraft(['build', '--target-arch=arm64'],
+                           'autotools-hello')
+        binary = os.path.join(self.parts_dir, 'make-project', 'install', 'bin',
+                              'test')
+        self.assertThat(binary, HasArchitecture('aarch64'))
