@@ -55,8 +55,28 @@ class PullPropertiesTestCase(integration_tests.TestCase):
         self.assertTrue(len(state.assets['stage-packages']) > 0)
         self.assertIn('build-packages', state.assets)
         self.assertTrue('stage-packages' in state.properties)
-        self.assertEqual('bar', state.properties['foo'])
-        self.assertEqual(['curl'], state.properties['stage-packages'])
+        self.assertThat(state.properties['foo'], Equals('bar'))
+        self.assertThat(state.properties['stage-packages'], Equals(['curl']))
+
+    def test_pull_with_arch(self):
+        self.run_snapcraft(['pull', '--target-arch=i386', 'go-hello'],
+                           'go-hello')
+        state_file = os.path.join(self.parts_dir,
+                                  'go-hello', 'state', 'pull')
+        self.assertThat(state_file, FileExists())
+        with open(state_file) as f:
+            state = yaml.load(f)
+        self.assertThat(state.project_options['deb_arch'], Equals('i386'))
+
+    def test_arch_with_pull(self):
+        self.run_snapcraft(['--target-arch=i386', 'pull', 'go-hello'],
+                           'go-hello')
+        state_file = os.path.join(self.parts_dir,
+                                  'go-hello', 'state', 'pull')
+        self.assertThat(state_file, FileExists())
+        with open(state_file) as f:
+            state = yaml.load(f)
+        self.assertThat(state.project_options['deb_arch'], Equals('i386'))
 
 
 class AssetTrackingTestCase(integration_tests.TestCase):
@@ -275,5 +295,6 @@ class SubversionAssetTrackingTestCase(integration_tests.TestCase):
             state = yaml.load(f)
 
         self.assertIn('source-details', state.assets)
-        self.assertEqual(expected_commit,
-                         state.assets['source-details']['source-commit'])
+        self.assertThat(
+            state.assets['source-details']['source-commit'],
+            Equals(expected_commit))
