@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -20,5 +20,4 @@ import integration_tests
 class WikiTestCase(integration_tests.TestCase):
 
     def test_pull_wiki_part(self):
-        project_dir = 'wiki'
-        self.run_snapcraft('pull', project_dir)
+        self.run_snapcraft('pull', 'wiki')
