@@ -13,11 +13,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-import sys
 import click
-
-from snapcraft.internal import parts
-from . import echo
+from snapcraft.internal import remote_parts
 
 
 @click.group(context_settings={})
@@ -30,7 +27,7 @@ def partscli(ctx):
 @click.pass_context
 def update(ctx):
     """Updates the parts listing from the cloud."""
-    parts.update()
+    remote_parts.update()
 
 
 @partscli.command()
@@ -44,11 +41,7 @@ def define(ctx, part):
         snapcraft define my-part1
 
     """
-    try:
-        parts.define(part)
-    except RuntimeError as e:
-        echo.error(e)
-        sys.exit(1)
+    remote_parts.define(part)
 
 
 @partscli.command()
@@ -62,4 +55,4 @@ def search(ctx, query):
         snapcraft search go
 
     """
-    parts.search(' '.join(query))
+    remote_parts.search(' '.join(query))
