@@ -29,8 +29,10 @@ import yaml
 
 from snapcraft import file_utils
 from snapcraft import shell_utils
-from snapcraft.internal import common
-from snapcraft.internal.errors import MissingGadgetError
+from snapcraft.internal import (
+    common,
+    errors
+)
 from snapcraft.internal.deprecations import handle_deprecation_notice
 from snapcraft.internal.sources import get_source_handler_from_type
 from snapcraft.internal.states import (
@@ -51,6 +53,7 @@ _MANDATORY_PACKAGE_KEYS = [
 _OPTIONAL_PACKAGE_KEYS = [
     'architectures',
     'assumes',
+    'base',
     'environment',
     'type',
     'plugs',
@@ -66,7 +69,7 @@ class CommandError(Exception):
     pass
 
 
-def create_snap_packaging(config_data, project_options):
+def create_snap_packaging(config_data, project_options, snapcraft_yaml_path):
     """Create snap.yaml and related assets in meta.
 
     Create the meta directory and provision it with snap.yaml in the snap dir
@@ -76,7 +79,8 @@ def create_snap_packaging(config_data, project_options):
     :param dict config_data: project values defined in snapcraft.yaml.
     :return: meta_dir.
     """
-    packaging = _SnapPackaging(config_data, project_options)
+    packaging = _SnapPackaging(
+        config_data, project_options, snapcraft_yaml_path)
     packaging.write_snap_yaml()
     packaging.setup_assets()
     packaging.generate_hook_wrappers()
@@ -91,7 +95,8 @@ class _SnapPackaging:
     def meta_dir(self):
         return self._meta_dir
 
-    def __init__(self, config_data, project_options):
+    def __init__(self, config_data, project_options, snapcraft_yaml_path):
+        self._snapcraft_yaml_path = snapcraft_yaml_path
         self._prime_dir = project_options.prime_dir
         self._parts_dir = project_options.parts_dir
         self._arch_triplet = project_options.arch_triplet
@@ -128,30 +133,36 @@ class _SnapPackaging:
 
         if self._config_data.get('type', '') == 'gadget':
             if not os.path.exists('gadget.yaml'):
-                raise MissingGadgetError()
+                raise errors.MissingGadgetError()
             file_utils.link_or_copy(
                 'gadget.yaml', os.path.join(self.meta_dir, 'gadget.yaml'))
 
-    def _record_snapcraft(self):
-        record_dir = os.path.join(self._prime_dir, 'snap')
-        record_file_path = os.path.join(record_dir, 'snapcraft.yaml')
-        if os.path.isfile(record_file_path):
-            os.unlink(record_file_path)
+    def _record_manifest_and_source_snapcraft_yaml(self):
+        prime_snap_dir = os.path.join(self._prime_dir, 'snap')
+        recorded_snapcraft_yaml_path = os.path.join(
+            prime_snap_dir, 'snapcraft.yaml')
+        if os.path.isfile(recorded_snapcraft_yaml_path):
+            os.unlink(recorded_snapcraft_yaml_path)
+        manifest_file_path = os.path.join(prime_snap_dir, 'manifest.yaml')
+        if os.path.isfile(manifest_file_path):
+            os.unlink(manifest_file_path)
 
         # FIXME hide this functionality behind a feature flag for now
         if os.environ.get('SNAPCRAFT_BUILD_INFO'):
-            os.makedirs(record_dir, exist_ok=True)
-            with open(record_file_path, 'w') as record_file:
-                annotated_snapcraft = self._annotate_snapcraft(
-                    copy.deepcopy(self._config_data))
-                yaml.dump(annotated_snapcraft, record_file)
+            os.makedirs(prime_snap_dir, exist_ok=True)
+            shutil.copy2(
+                self._snapcraft_yaml_path, recorded_snapcraft_yaml_path)
+            annotated_snapcraft = self._annotate_snapcraft(
+                copy.deepcopy(self._config_data))
+            with open(manifest_file_path, 'w') as manifest_file:
+                yaml.dump(annotated_snapcraft, manifest_file)
 
     def _annotate_snapcraft(self, data):
         data['build-packages'] = get_global_state().assets.get(
             'build-packages', [])
         for part in data['parts']:
-            pull_state = get_state(
-                os.path.join(self._parts_dir, part, 'state'), 'pull')
+            state_dir = os.path.join(self._parts_dir, part, 'state')
+            pull_state = get_state(state_dir, 'pull')
             data['parts'][part]['build-packages'] = (
                 pull_state.assets.get('build-packages', []))
             data['parts'][part]['stage-packages'] = (
@@ -159,6 +170,8 @@ class _SnapPackaging:
             source_details = pull_state.assets.get('source-details', {})
             if source_details:
                 data['parts'][part].update(source_details)
+            build_state = get_state(state_dir, 'build')
+            data['parts'][part].update(build_state.assets)
         return data
 
     def write_snap_directory(self):
@@ -199,7 +212,7 @@ class _SnapPackaging:
 
                     file_utils.link_or_copy(source, destination)
 
-        self._record_snapcraft()
+        self._record_manifest_and_source_snapcraft_yaml()
 
     def generate_hook_wrappers(self):
         snap_hooks_dir = os.path.join(self._prime_dir, 'snap', 'hooks')
@@ -259,6 +272,7 @@ class _SnapPackaging:
                 snap_yaml[key_name] = self._config_data[key_name]
 
         if 'apps' in self._config_data:
+            _verify_app_paths(basedir='prime', apps=self._config_data['apps'])
             snap_yaml['apps'] = self._wrap_apps(self._config_data['apps'])
 
         return snap_yaml
@@ -377,9 +391,7 @@ class _SnapPackaging:
             try:
                 app[k] = self._wrap_exe(app[k], '{}-{}'.format(k, name))
             except CommandError as e:
-                raise EnvironmentError(
-                    'The specified command {!r} defined in the app {!r} '
-                    'does not exist or is not executable'.format(str(e), name))
+                raise errors.InvalidAppCommandError(str(e), name)
         desktop_file_name = app.pop('desktop', '')
         if desktop_file_name:
             desktop_file = _DesktopFile(
@@ -398,9 +410,9 @@ class _DesktopFile:
         self._prime_dir = prime_dir
         self._path = os.path.join(prime_dir, filename)
         if not os.path.exists(self._path):
-            raise EnvironmentError(
-                'The specified desktop file {!r} defined in the app '
-                '{!r} does not exist'.format(filename, name))
+            raise errors.InvalidDesktopFileError(
+                filename, 'does not exist (defined in the app {!r})'.format(
+                    name))
 
     def parse_and_reformat(self):
         self._parser = configparser.ConfigParser(interpolation=None)
@@ -408,13 +420,11 @@ class _DesktopFile:
         self._parser.read(self._path, encoding='utf-8')
         section = 'Desktop Entry'
         if section not in self._parser.sections():
-            raise EnvironmentError(
-                'The specified desktop file {!r} is not a valid '
-                'desktop file'.format(self._filename))
+            raise errors.InvalidDesktopFileError(
+                self._filename, "missing 'Desktop Entry' section")
         if 'Exec' not in self._parser[section]:
-            raise EnvironmentError(
-                'The specified desktop file {!r} is missing the '
-                '"Exec" key'.format(self._filename))
+            raise errors.InvalidDesktopFileError(
+                self._filename, "missing 'Exec' key")
         # XXX: do we want to allow more parameters for Exec?
         if self._name == self._snap_name:
             exec_value = '{} %U'.format(self._name)
@@ -458,3 +468,14 @@ def _validate_hook(hook_path):
     if not os.stat(hook_path).st_mode & stat.S_IEXEC:
         asset = os.path.basename(hook_path)
         raise CommandError('hook {!r} is not executable'.format(asset))
+
+
+def _verify_app_paths(basedir, apps):
+    for app in apps:
+        path_entries = [i for i in ('desktop', 'completer')
+                        if i in apps[app]]
+        for path_entry in path_entries:
+            file_path = os.path.join(basedir, apps[app][path_entry])
+            if not os.path.exists(file_path):
+                raise errors.SnapcraftPathEntryError(
+                    app=app, key=path_entry, value=file_path)
