@@ -84,8 +84,8 @@ class BaseRepo:
         - zip
 
         :param str source_type: a VCS source type to handle.
-        :returns: a list of packages that need to be installed on the host.
-        :rtype: list of strings.
+        :returns: a set of packages that need to be installed on the host.
+        :rtype: set of strings.
         """
         raise NotImplementedError()
 
@@ -100,6 +100,15 @@ class BaseRepo:
         """
         raise NotImplementedError()
 
+    @classmethod
+    def build_package_is_valid(cls, package_name):
+        """Check that a given package is valid on the host.
+
+        :param package_name: a package name to check.
+        :type package_name: str
+        """
+        raise NotImplementedError()
+
     @classmethod
     def is_package_installed(cls, package_name):
         """Return a bool indicating if package_name is installed.
