@@ -19,7 +19,6 @@ import glob
 import hashlib
 import logging
 import os
-import platform
 import shutil
 import stat
 import string
@@ -147,7 +146,7 @@ class _AptCache:
 
     def _collected_sources_list(self):
         if self._use_geoip or self._sources_list:
-            release = platform.linux_distribution()[2]
+            release = common.get_os_release_info()['VERSION_CODENAME']
             return _format_sources_list(
                 self._sources_list, deb_arch=self._deb_arch,
                 use_geoip=self._use_geoip, release=release)
@@ -171,17 +170,17 @@ class Ubuntu(BaseRepo):
     @classmethod
     def get_packages_for_source_type(cls, source_type):
         if source_type == 'bzr':
-            packages = 'bzr'
+            packages = {'bzr'}
         elif source_type == 'git':
-            packages = 'git'
+            packages = {'git'}
         elif source_type == 'tar':
-            packages = 'tar'
+            packages = {'tar'}
         elif source_type == 'hg' or source_type == 'mercurial':
-            packages = 'mercurial'
+            packages = {'mercurial'}
         elif source_type == 'subversion' or source_type == 'svn':
-            packages = 'subversion'
+            packages = {'subversion'}
         else:
-            packages = []
+            packages = set()
 
         return packages
 
@@ -197,7 +196,7 @@ class Ubuntu(BaseRepo):
             try:
                 cls._mark_install(apt_cache, package_names)
             except errors.PackageNotFoundError as e:
-                raise errors.BuildPackageNotFoundError(e)
+                raise errors.BuildPackageNotFoundError(e.package_name)
             for package in apt_cache.get_changes():
                 new_packages.append((package.name, package.candidate.version))
 
@@ -251,6 +250,11 @@ class Ubuntu(BaseRepo):
                 'Impossible to mark packages as auto-installed: {}'
                 .format(e))
 
+    @classmethod
+    def build_package_is_valid(cls, package_name):
+        with apt.Cache() as apt_cache:
+            return package_name in apt_cache
+
     @classmethod
     def is_package_installed(cls, package_name):
         with apt.Cache() as apt_cache:
@@ -259,7 +263,6 @@ class Ubuntu(BaseRepo):
     def __init__(self, rootdir, sources=None, project_options=None):
         super().__init__(rootdir)
         self._downloaddir = os.path.join(rootdir, 'download')
-        os.makedirs(self._downloaddir, exist_ok=True)
 
         if not project_options:
             project_options = snapcraft.ProjectOptions()
@@ -437,11 +440,6 @@ def _try_copy_local(path, target):
         return False
 
 
-def check_for_command(command):
-    if not shutil.which(command):
-        raise errors.MissingCommandError([command])
-
-
 def _set_pkg_version(pkg, version):
     """Set cadidate version to a specific version if available"""
     if version in pkg.versions:
