@@ -39,11 +39,17 @@ def _schema_properties():
 class BuildState(PartState):
     yaml_tag = u'!BuildState'
 
-    def __init__(self, property_names, part_properties=None, project=None):
+    def __init__(
+            self, property_names, part_properties=None, project=None,
+            plugin_assets=None):
         # Save this off before calling super() since we'll need it
         # FIXME: for 3.x the name `schema_properties` is leaking
         #        implementation details from a higher layer.
         self.schema_properties = property_names
+        if plugin_assets:
+            self.assets = plugin_assets
+        else:
+            self.assets = {}
 
         super().__init__(part_properties, project)
 
