@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -19,7 +19,7 @@ import copy
 from unittest import mock
 
 import fixtures
-from testtools.matchers import HasLength
+from testtools.matchers import Equals, HasLength
 
 import snapcraft
 from snapcraft import tests
@@ -48,13 +48,13 @@ class AntPluginTestCase(tests.TestCase):
                 'Expected {!r} to be included in properties'.format(expected))
 
         properties_type = schema['properties']['ant-properties']['type']
-        self.assertEqual(properties_type, 'object',
-                         'Expected "ant-properties" "type" to be "object", '
-                         'but it was "{}"'.format(properties_type))
+        self.assertThat(properties_type, Equals('object'),
+                        'Expected "ant-properties" "type" to be "object", '
+                        'but it was "{}"'.format(properties_type))
         build_targets_type = schema['properties']['ant-build-targets']['type']
-        self.assertEqual(build_targets_type, 'array',
-                         'Expected "ant-build-targets" "type" to be "object", '
-                         'but it was "{}"'.format(build_targets_type))
+        self.assertThat(build_targets_type, Equals('array'),
+                        'Expected "ant-build-targets" "type" to be "object", '
+                        'but it was "{}"'.format(build_targets_type))
 
     def test_get_build_properties(self):
         expected_build_properties = ['ant-build-targets', 'ant-properties']
@@ -100,9 +100,9 @@ class AntPluginTestCase(tests.TestCase):
         destination = '-Ddist.dir={}'.format(plugin.installdir)
         basedir = '-Dbasedir=.'
         args = run_mock.call_args[0][0]
-        self.assertEqual(args[0], 'ant')
-        self.assertEqual(args[1], 'artifacts')
-        self.assertEqual(args[2], 'jar')
+        self.assertThat(args[0], Equals('ant'))
+        self.assertThat(args[1], Equals('artifacts'))
+        self.assertThat(args[2], Equals('jar'))
         self.assertIn(destination, args)
         self.assertIn(basedir, args)
 
@@ -123,11 +123,11 @@ class AntPluginTestCase(tests.TestCase):
 
     def test_env_proxies(self):
         env_vars = (
-            ('http_proxy', 'http://localhost:3132'),
-            ('https_proxy', 'http://localhost2:3133'),
+            ('http_proxy', 'http://user:pass@localhost:3132'),
+            ('https_proxy', 'http://user2:pass2@localhost2:3133'),
         )
-        for v in env_vars:
-            self.useFixture(fixtures.EnvironmentVariable(v[0], v[1]))
+        for key, value in env_vars:
+            self.useFixture(fixtures.EnvironmentVariable(key, value))
         plugin = ant.AntPlugin('test-part', self.options,
                                self.project_options)
 
@@ -135,5 +135,7 @@ class AntPluginTestCase(tests.TestCase):
         self.assertIn(
             "ANT_OPTS='"
             "-Dhttp.proxyHost=localhost -Dhttp.proxyPort=3132 "
-            "-Dhttps.proxyHost=localhost2 -Dhttps.proxyPort=3133'",
+            "-Dhttp.proxyUser=user -Dhttp.proxyPassword=pass "
+            "-Dhttps.proxyHost=localhost2 -Dhttps.proxyPort=3133 "
+            "-Dhttps.proxyUser=user2 -Dhttps.proxyPassword=pass2'",
             env)
