@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -17,6 +17,7 @@
 import builtins
 import os
 import os.path
+import re
 import subprocess
 import shutil
 
@@ -32,6 +33,7 @@ from testtools.matchers import (
 
 import snapcraft
 from snapcraft.plugins import catkin
+from snapcraft.plugins import _ros
 from snapcraft import (
     repo,
     tests,
@@ -45,10 +47,10 @@ class _CompareContainers():
         self.expected = expected
 
     def __eq__(self, container):
-        self.test.assertEqual(len(container), len(self.expected),
-                              'Expected {} items to be in container, '
-                              'got {}'.format(len(self.expected),
-                                              len(container)))
+        self.test.assertThat(len(container), Equals(len(self.expected)),
+                             'Expected {} items to be in container, '
+                             'got {}'.format(len(self.expected),
+                                             len(container)))
 
         for expectation in self.expected:
             self.test.assertTrue(expectation in container,
@@ -65,12 +67,15 @@ class CatkinPluginBaseTestCase(tests.TestCase):
 
         class props:
             rosdistro = 'indigo'
+            ubuntu_distro = 'trusty'
             catkin_packages = ['my_package']
             source_space = 'src'
             source_subdir = None
             include_roscore = False
+            catkin_cmake_args = []
             underlay = None
             rosinstall_files = None
+            build_attributes = []
 
         self.properties = props()
         self.project_options = snapcraft.ProjectOptions()
@@ -84,7 +89,8 @@ class CatkinPluginBaseTestCase(tests.TestCase):
         self.dependencies_mock = patcher.start()
         self.addCleanup(patcher.stop)
 
-        patcher = mock.patch('snapcraft.plugins.catkin._Rosdep')
+        patcher = mock.patch(
+            'snapcraft.plugins._ros.rosdep.Rosdep')
         self.rosdep_mock = patcher.start()
         self.addCleanup(patcher.stop)
 
@@ -97,10 +103,15 @@ class CatkinPluginBaseTestCase(tests.TestCase):
         self.addCleanup(patcher.stop)
 
     def assert_rosdep_setup(self, rosdistro, package_path, rosdep_path,
-                            sources):
+                            ubuntu_distro, sources):
         self.rosdep_mock.assert_has_calls([
-            mock.call(rosdistro, package_path, rosdep_path, sources,
-                      self.project_options),
+            mock.call(
+                ros_distro=rosdistro,
+                ros_package_path=package_path,
+                rosdep_path=rosdep_path,
+                ubuntu_distro=ubuntu_distro,
+                ubuntu_sources=sources,
+                project=self.project_options),
             mock.call().setup()])
 
     def assert_wstool_setup(self, package_path, wstool_path, sources):
@@ -117,13 +128,17 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
 
         properties = schema['properties']
         expected = ('rosdistro', 'catkin-packages', 'source-space',
-                    'include-roscore', 'underlay', 'rosinstall-files')
+                    'include-roscore', 'catkin-cmake-args', 'underlay',
+                    'rosinstall-files')
         self.assertThat(properties, HasLength(len(expected)))
         for prop in expected:
             self.assertThat(properties, Contains(prop))
 
+    def test_schema_rosdistro(self):
+        schema = catkin.CatkinPlugin.schema()
+
         # Check rosdistro property
-        rosdistro = properties['rosdistro']
+        rosdistro = schema['properties']['rosdistro']
         expected = ('type', 'default')
         self.assertThat(rosdistro, HasLength(len(expected)))
         for prop in expected:
@@ -131,8 +146,11 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
         self.assertThat(rosdistro['type'], Equals('string'))
         self.assertThat(rosdistro['default'], Equals('indigo'))
 
+    def test_schema_catkin_packages(self):
+        schema = catkin.CatkinPlugin.schema()
+
         # Check catkin-packages property
-        catkin_packages = properties['catkin-packages']
+        catkin_packages = schema['properties']['catkin-packages']
         expected = ('type', 'default', 'minitems', 'uniqueItems', 'items')
         self.assertThat(catkin_packages, HasLength(len(expected)))
         for prop in expected:
@@ -144,8 +162,16 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
         self.assertThat(catkin_packages['items'], Contains('type'))
         self.assertThat(catkin_packages['items']['type'], Equals('string'))
 
+        # Check required
+        self.assertTrue('catkin-packages' in schema['required'],
+                        'Expected "catkin-packages" to be included in '
+                        '"required"')
+
+    def test_schema_source_space(self):
+        schema = catkin.CatkinPlugin.schema()
+
         # Check source-space property
-        source_space = properties['source-space']
+        source_space = schema['properties']['source-space']
         expected = ('type', 'default')
         self.assertThat(source_space, HasLength(len(expected)))
         for prop in expected:
@@ -153,17 +179,38 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
         self.assertThat(source_space['type'], Equals('string'))
         self.assertThat(source_space['default'], Equals('src'))
 
+    def test_schema_include_roscore(self):
+        schema = catkin.CatkinPlugin.schema()
+
         # Check include-roscore property
-        include_roscore = properties['include-roscore']
+        include_roscore = schema['properties']['include-roscore']
         expected = ('type', 'default')
         self.assertThat(include_roscore, HasLength(len(expected)))
         for prop in expected:
             self.assertThat(include_roscore, Contains(prop))
         self.assertThat(include_roscore['type'], Equals('boolean'))
-        self.assertThat(include_roscore['default'], Equals('true'))
+        self.assertThat(include_roscore['default'], Equals(True))
+
+    def test_schema_catkin_catkin_cmake_args(self):
+        schema = catkin.CatkinPlugin.schema()
+
+        # Check catkin-cmake-args property
+        catkin_cmake_args = schema['properties']['catkin-cmake-args']
+        expected = ('type', 'default', 'minitems', 'items')
+        self.assertThat(catkin_cmake_args, HasLength(len(expected)))
+        for prop in expected:
+            self.assertThat(catkin_cmake_args, Contains(prop))
+        self.assertThat(catkin_cmake_args['type'], Equals('array'))
+        self.assertThat(catkin_cmake_args['default'], Equals([]))
+        self.assertThat(catkin_cmake_args['minitems'], Equals(1))
+        self.assertThat(catkin_cmake_args['items'], Contains('type'))
+        self.assertThat(catkin_cmake_args['items']['type'], Equals('string'))
+
+    def test_schema_underlay(self):
+        schema = catkin.CatkinPlugin.schema()
 
         # Check underlay property
-        underlay = properties['underlay']
+        underlay = schema['properties']['underlay']
         expected = ('type', 'properties', 'required')
         self.assertThat(underlay, HasLength(len(expected)))
         for prop in expected:
@@ -188,35 +235,43 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
         self.assertThat(underlay_run_path, Contains('type'))
         self.assertThat(underlay_run_path['type'], Equals('string'))
 
+    def test_schema_rosinstall_files(self):
+        schema = catkin.CatkinPlugin.schema()
+
         # Check rosinstall-files property
-        rosinstall_files = properties['rosinstall-files']
+        rosinstall_files = schema['properties']['rosinstall-files']
         expected = ('type', 'default', 'minitems', 'uniqueItems', 'items')
         self.assertThat(rosinstall_files, HasLength(len(expected)))
         for prop in expected:
             self.assertThat(rosinstall_files, Contains(prop))
-        self.assertThat(catkin_packages['type'], Equals('array'))
-        self.assertThat(catkin_packages['default'], Equals([]))
-        self.assertThat(catkin_packages['minitems'], Equals(1))
-        self.assertTrue(catkin_packages['uniqueItems'])
-        self.assertThat(catkin_packages['items'], Contains('type'))
-        self.assertThat(catkin_packages['items']['type'], Equals('string'))
-
-        # Check required
-        self.assertTrue('catkin-packages' in schema['required'],
-                        'Expected "catkin-packages" to be included in '
-                        '"required"')
+        self.assertThat(rosinstall_files['type'], Equals('array'))
+        self.assertThat(rosinstall_files['default'], Equals([]))
+        self.assertThat(rosinstall_files['minitems'], Equals(1))
+        self.assertTrue(rosinstall_files['uniqueItems'])
+        self.assertThat(rosinstall_files['items'], Contains('type'))
+        self.assertThat(rosinstall_files['items']['type'], Equals('string'))
 
     def test_get_pull_properties(self):
         expected_pull_properties = ['rosdistro', 'catkin-packages',
                                     'source-space', 'include-roscore',
-                                    'underlay']
-        resulting_pull_properties = catkin.CatkinPlugin.get_pull_properties()
+                                    'underlay', 'rosinstall-files']
+        actual_pull_properties = catkin.CatkinPlugin.get_pull_properties()
 
-        self.assertThat(resulting_pull_properties,
+        self.assertThat(actual_pull_properties,
                         HasLength(len(expected_pull_properties)))
 
         for property in expected_pull_properties:
-            self.assertIn(property, resulting_pull_properties)
+            self.assertIn(property, actual_pull_properties)
+
+    def test_get_build_properties(self):
+        expected_build_properties = ['build-attributes', 'catkin-cmake-args']
+        actual_build_properties = catkin.CatkinPlugin.get_build_properties()
+
+        self.assertThat(actual_build_properties,
+                        HasLength(len(expected_build_properties)))
+
+        for property in expected_build_properties:
+            self.assertIn(property, actual_build_properties)
 
     def test_invalid_rosdistro(self):
         self.properties.rosdistro = 'invalid'
@@ -226,9 +281,11 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
             'test-part', self.properties,
             self.project_options)
 
-        self.assertEqual(str(raised),
-                         "Unsupported rosdistro: 'invalid'. The supported ROS "
-                         "distributions are 'indigo', 'jade', and 'kinetic'")
+        self.assertThat(
+            str(raised),
+            Equals("Unsupported rosdistro: 'invalid'. The supported ROS "
+                   "distributions are 'indigo', 'jade', 'kinetic', and "
+                   "'lunar'"))
 
     def test_get_stage_sources_indigo(self):
         self.properties.rosdistro = 'indigo'
@@ -248,13 +305,19 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
                                      self.project_options)
         self.assertTrue('xenial' in plugin.PLUGIN_STAGE_SOURCES)
 
+    def test_get_stage_sources_lunar(self):
+        self.properties.rosdistro = 'lunar'
+        plugin = catkin.CatkinPlugin('test-part', self.properties,
+                                     self.project_options)
+        self.assertTrue('xenial' in plugin.PLUGIN_STAGE_SOURCES)
+
     @mock.patch('snapcraft.plugins.catkin._Compilers')
     def test_pull_invalid_dependency(self, compilers_mock):
         plugin = catkin.CatkinPlugin('test-part', self.properties,
                                      self.project_options)
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
-        self.dependencies_mock.return_value = ['foo']
+        self.dependencies_mock.return_value = {'apt': {'foo'}}
 
         mock_instance = self.ubuntu_mock.return_value
         mock_instance.get.side_effect = repo.errors.PackageNotFoundError('foo')
@@ -263,9 +326,9 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
             RuntimeError,
             plugin.pull)
 
-        self.assertEqual(str(raised),
-                         'Failed to fetch system dependencies: The '
-                         "package 'foo' was not found.")
+        self.assertThat(str(raised),
+                        Equals('Failed to fetch system dependencies: The '
+                               "package 'foo' was not found."))
 
     def test_pull_unable_to_resolve_roscore(self):
         self.properties.include_roscore = True
@@ -274,14 +337,15 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
         # No system dependencies
-        self.dependencies_mock.return_value = set()
+        self.dependencies_mock.return_value = {}
 
         self.rosdep_mock.return_value.resolve_dependency.return_value = None
 
         raised = self.assertRaises(RuntimeError, plugin.pull)
 
-        self.assertEqual(str(raised),
-                         'Unable to determine system dependency for roscore')
+        self.assertThat(
+            str(raised),
+            Equals('Unable to determine system dependency for roscore'))
 
     @mock.patch.object(catkin.CatkinPlugin, '_generate_snapcraft_setup_sh')
     def test_pull_invalid_underlay(self, generate_setup_mock):
@@ -294,7 +358,7 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
         # No system dependencies
-        self.dependencies_mock.return_value = set()
+        self.dependencies_mock.return_value = {}
 
         raised = self.assertRaises(
             errors.SnapcraftEnvironmentError, plugin.pull)
@@ -309,7 +373,7 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
                                      self.project_options)
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
-        self.dependencies_mock.return_value = {'foo', 'bar', 'baz'}
+        self.dependencies_mock.return_value = {'apt': {'foo', 'bar', 'baz'}}
 
         plugin.pull()
         os.makedirs(plugin._rosdep_path)
@@ -334,10 +398,10 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
                                      self.project_options)
         raised = self.assertRaises(FileNotFoundError, plugin.pull)
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Unable to find package path: "{}"'.format(os.path.join(
-                plugin.sourcedir, 'src')))
+            Equals('Unable to find package path: "{}"'.format(os.path.join(
+                plugin.sourcedir, 'src'))))
 
     def test_valid_catkin_workspace_source_space(self):
         self.properties.source_space = 'foo'
@@ -362,10 +426,10 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
                                      self.project_options)
         raised = self.assertRaises(FileNotFoundError, plugin.pull)
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Unable to find package path: "{}"'.format(os.path.join(
-                plugin.sourcedir, self.properties.source_space)))
+            Equals('Unable to find package path: "{}"'.format(os.path.join(
+                plugin.sourcedir, self.properties.source_space))))
 
     def test_invalid_catkin_workspace_source_space_same_as_source(self):
         self.properties.source_space = '.'
@@ -379,48 +443,9 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
             'test-part', self.properties,
             self.project_options)
 
-        self.assertEqual(str(raised),
-                         'source-space cannot be the root of the Catkin '
-                         'workspace')
-
-    @mock.patch('snapcraft.plugins.catkin._Compilers')
-    @mock.patch.object(catkin.CatkinPlugin, 'run')
-    @mock.patch.object(catkin.CatkinPlugin, '_run_in_bash')
-    @mock.patch.object(catkin.CatkinPlugin, 'run_output', return_value='foo')
-    @mock.patch.object(catkin.CatkinPlugin, '_prepare_build')
-    @mock.patch.object(catkin.CatkinPlugin, '_finish_build')
-    def test_build(self, finish_build_mock, prepare_build_mock,
-                   run_output_mock, bashrun_mock, run_mock, compilers_mock):
-        plugin = catkin.CatkinPlugin('test-part', self.properties,
-                                     self.project_options)
-        os.makedirs(os.path.join(plugin.sourcedir, 'src'))
-
-        plugin.build()
-
-        prepare_build_mock.assert_called_once_with()
-
-        # Matching like this for order independence (otherwise it would be
-        # quite fragile)
-        class check_build_command():
-            def __eq__(self, args):
-                command = ' '.join(args)
-                return (
-                    args[0] == 'catkin_make_isolated' and
-                    '--install' in command and
-                    '--pkg my_package' in command and
-                    '--directory {}'.format(plugin.builddir) in command and
-                    '--install-space {}'.format(plugin.rosdir) in command and
-                    '--source-space {}'.format(os.path.join(
-                        plugin.builddir,
-                        plugin.options.source_space)) in command)
-
-        bashrun_mock.assert_called_with(check_build_command(), env=mock.ANY)
-
-        self.assertFalse(
-            self.dependencies_mock.called,
-            'Dependencies should have been discovered in the pull() step')
-
-        finish_build_mock.assert_called_once_with()
+        self.assertThat(
+            str(raised),
+            Equals('source-space cannot be the root of the Catkin workspace'))
 
     @mock.patch('snapcraft.plugins.catkin._Compilers')
     @mock.patch.object(catkin.CatkinPlugin, 'run')
@@ -546,9 +571,9 @@ class CatkinPluginTestCase(CatkinPluginBaseTestCase):
 
         # Verify that the absolute path in 10.ros.sh was rewritten correctly
         with open(ros_profile, 'r') as f:
-            self.assertEqual(f.read(), 'python foo',
-                             'The absolute path to python was not replaced as '
-                             'expected')
+            self.assertThat(f.read(), Equals('python foo'),
+                            'The absolute path to python was not replaced as '
+                            'expected')
 
     def _verify_run_environment(self, plugin):
         python_path = os.path.join(
@@ -779,7 +804,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
                                      self.project_options)
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
-        self.dependencies_mock.return_value = {'foo', 'bar', 'baz'}
+        self.dependencies_mock.return_value = {'apt': {'foo', 'bar', 'baz'}}
 
         plugin.pull()
 
@@ -787,6 +812,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
             self.properties.rosdistro,
             os.path.join(plugin.sourcedir, 'src'),
             os.path.join(plugin.partdir, 'rosdep'),
+            self.properties.ubuntu_distro,
             plugin.PLUGIN_STAGE_SOURCES)
 
         self.wstool_mock.assert_not_called()
@@ -801,9 +827,10 @@ class PullTestCase(CatkinPluginBaseTestCase):
         # Verify that dependencies were found as expected. TODO: Would really
         # like to use ANY here instead of verifying explicit arguments, but
         # Python issue #25195 won't let me.
-        self.assertEqual(1, self.dependencies_mock.call_count)
-        self.assertEqual({'my_package'},
-                         self.dependencies_mock.call_args[0][0])
+        self.assertThat(self.dependencies_mock.call_count, Equals(1))
+        self.assertThat(
+            self.dependencies_mock.call_args[0][0],
+            Equals({'my_package'}))
 
         # Verify that the dependencies were installed
         self.ubuntu_mock.return_value.get.assert_called_with(
@@ -820,7 +847,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
         # No system dependencies (only local)
-        self.dependencies_mock.return_value = set()
+        self.dependencies_mock.return_value = {}
 
         plugin.pull()
 
@@ -828,6 +855,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
             self.properties.rosdistro,
             os.path.join(plugin.sourcedir, 'src'),
             os.path.join(plugin.partdir, 'rosdep'),
+            self.properties.ubuntu_distro,
             plugin.PLUGIN_STAGE_SOURCES)
 
         self.wstool_mock.assert_not_called()
@@ -842,9 +870,10 @@ class PullTestCase(CatkinPluginBaseTestCase):
         # Verify that dependencies were found as expected. TODO: Would really
         # like to use ANY here instead of verifying explicit arguments, but
         # Python issue #25195 won't let me.
-        self.assertEqual(1, self.dependencies_mock.call_count)
-        self.assertEqual({'my_package', 'package_2'},
-                         self.dependencies_mock.call_args[0][0])
+        self.assertThat(self.dependencies_mock.call_count, Equals(1))
+        self.assertThat(
+            self.dependencies_mock.call_args[0][0],
+            Equals({'my_package', 'package_2'}))
 
         # Verify that no .deb packages were installed
         self.assertTrue(mock.call().unpack(plugin.installdir) not in
@@ -858,11 +887,11 @@ class PullTestCase(CatkinPluginBaseTestCase):
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
         # No system dependencies
-        self.dependencies_mock.return_value = set()
+        self.dependencies_mock.return_value = {}
 
         def resolve(package_name):
             if package_name == 'ros_core':
-                return ['ros-core-dependency']
+                return {'apt': {'ros-core-dependency'}}
 
         self.rosdep_mock.return_value.resolve_dependency = resolve
 
@@ -872,6 +901,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
             self.properties.rosdistro,
             os.path.join(plugin.sourcedir, 'src'),
             os.path.join(plugin.partdir, 'rosdep'),
+            self.properties.ubuntu_distro,
             plugin.PLUGIN_STAGE_SOURCES)
 
         self.wstool_mock.assert_not_called()
@@ -897,7 +927,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
         os.makedirs(os.path.join(plugin.sourcedir, 'src'))
 
         # No system dependencies
-        self.dependencies_mock.return_value = set()
+        self.dependencies_mock.return_value = {}
 
         plugin.pull()
 
@@ -905,6 +935,7 @@ class PullTestCase(CatkinPluginBaseTestCase):
             self.properties.rosdistro,
             os.path.join(plugin.sourcedir, 'src'),
             os.path.join(plugin.partdir, 'rosdep'),
+            self.properties.ubuntu_distro,
             plugin.PLUGIN_STAGE_SOURCES)
 
         self.assert_wstool_setup(
@@ -930,6 +961,89 @@ class PullTestCase(CatkinPluginBaseTestCase):
                         self.ubuntu_mock.mock_calls)
 
 
+class BuildTestCase(CatkinPluginBaseTestCase):
+
+    scenarios = [
+        ('release without catkin-cmake-args', {
+            'build_attributes': [],
+            'catkin_cmake_args': [],
+        }),
+        ('release with catkin-cmake-args', {
+            'build_attributes': [],
+            'catkin_cmake_args': ['-DFOO'],
+        }),
+        ('debug without catkin-cmake-args', {
+            'build_attributes': ['debug'],
+            'catkin_cmake_args': [],
+        }),
+        ('debug with catkin-cmake-args', {
+            'build_attributes': ['debug'],
+            'catkin_cmake_args': ['-DFOO'],
+        })
+    ]
+
+    def setUp(self):
+        super().setUp()
+        self.properties.build_attributes.extend(self.build_attributes)
+        self.properties.catkin_cmake_args = self.catkin_cmake_args
+
+    @mock.patch('snapcraft.plugins.catkin._Compilers')
+    @mock.patch.object(catkin.CatkinPlugin, 'run')
+    @mock.patch.object(catkin.CatkinPlugin, '_run_in_bash')
+    @mock.patch.object(catkin.CatkinPlugin, 'run_output', return_value='foo')
+    @mock.patch.object(catkin.CatkinPlugin, '_prepare_build')
+    @mock.patch.object(catkin.CatkinPlugin, '_finish_build')
+    def test_build(self, finish_build_mock, prepare_build_mock,
+                   run_output_mock, bashrun_mock, run_mock, compilers_mock):
+        plugin = catkin.CatkinPlugin('test-part', self.properties,
+                                     self.project_options)
+        os.makedirs(os.path.join(plugin.sourcedir, 'src'))
+
+        plugin.build()
+
+        prepare_build_mock.assert_called_once_with()
+
+        # Matching like this for order independence (otherwise it would be
+        # quite fragile)
+        build_attributes = self.build_attributes
+        catkin_cmake_args = self.catkin_cmake_args
+
+        class check_build_command():
+            def __eq__(self, args):
+                command = ' '.join(args)
+                if 'debug' in build_attributes:
+                    build_type_valid = re.match(
+                        '.*--cmake-args.*-DCMAKE_BUILD_TYPE=Debug', command)
+                else:
+                    build_type_valid = re.match(
+                        '.*--cmake-args.*-DCMAKE_BUILD_TYPE=Release', command)
+                args_valid = True
+                if catkin_cmake_args:
+                    expected_args = ' '.join(catkin_cmake_args)
+                    args_valid = re.match(
+                        '.*--cmake-args.*{}'.format(re.escape(expected_args)),
+                        command)
+                return (
+                    args_valid and
+                    build_type_valid and
+                    args[0] == 'catkin_make_isolated' and
+                    '--install' in command and
+                    '--pkg my_package' in command and
+                    '--directory {}'.format(plugin.builddir) in command and
+                    '--install-space {}'.format(plugin.rosdir) in command and
+                    '--source-space {}'.format(os.path.join(
+                        plugin.builddir,
+                        plugin.options.source_space)) in command)
+
+        bashrun_mock.assert_called_with(check_build_command(), env=mock.ANY)
+
+        self.assertFalse(
+            self.dependencies_mock.called,
+            'Dependencies should have been discovered in the pull() step')
+
+        finish_build_mock.assert_called_once_with()
+
+
 class FinishBuildTestCase(CatkinPluginBaseTestCase):
 
     scenarios = [
@@ -1040,8 +1154,8 @@ class FinishBuildTestCase(CatkinPluginBaseTestCase):
         expected = 'CMAKE_PREFIX_PATH = []\n'
 
         with open(setup_file, 'r') as f:
-            self.assertEqual(
-                f.read(), expected,
+            self.assertThat(
+                f.read(), Equals(expected),
                 'The absolute path to python or the CMAKE_PREFIX_PATH '
                 'was not replaced as expected')
 
@@ -1058,10 +1172,13 @@ class FindSystemDependenciesTestCase(tests.TestCase):
         self.catkin_mock.find.side_effect = exception
 
     def test_find_system_dependencies_system_only(self):
-        self.rosdep_mock.resolve_dependency.return_value = ['baz']
+        self.rosdep_mock.resolve_dependency.return_value = {
+            'apt': {'baz'},
+        }
 
         self.assertThat(catkin._find_system_dependencies(
-            {'foo'}, self.rosdep_mock, self.catkin_mock), Equals({'baz'}))
+            {'foo'}, self.rosdep_mock, self.catkin_mock), Equals(
+            {'apt': {'baz'}}))
 
         self.rosdep_mock.get_dependencies.assert_called_once_with('foo')
         self.rosdep_mock.resolve_dependency.assert_called_once_with('bar')
@@ -1090,7 +1207,9 @@ class FindSystemDependenciesTestCase(tests.TestCase):
 
     def test_find_system_dependencies_mixed(self):
         self.rosdep_mock.get_dependencies.return_value = ['bar', 'baz', 'qux']
-        self.rosdep_mock.resolve_dependency.return_value = ['quux']
+        self.rosdep_mock.resolve_dependency.return_value = {
+            'apt': {'quux'},
+        }
 
         def _fake_find(package_name):
             if package_name == 'qux':
@@ -1100,7 +1219,7 @@ class FindSystemDependenciesTestCase(tests.TestCase):
         self.catkin_mock.find.side_effect = _fake_find
         self.assertThat(catkin._find_system_dependencies(
             {'foo', 'bar'}, self.rosdep_mock, self.catkin_mock),
-            Equals({'quux'}))
+            Equals({'apt': {'quux'}}))
 
         self.rosdep_mock.get_dependencies.assert_has_calls(
             [mock.call('foo'), mock.call('bar')], any_order=True)
@@ -1111,20 +1230,34 @@ class FindSystemDependenciesTestCase(tests.TestCase):
     def test_find_system_dependencies_missing_local_dependency(self):
         # Setup a dependency on a non-existing package, and it doesn't resolve
         # to a system dependency.'
-        exception = catkin.SystemDependencyNotFoundError('foo')
+        exception = _ros.rosdep.RosdepDependencyNotFoundError('foo')
         self.rosdep_mock.resolve_dependency.side_effect = exception
 
         raised = self.assertRaises(
-            RuntimeError,
+            catkin.CatkinInvalidSystemDependencyError,
             catkin._find_system_dependencies,
             {'foo'}, self.rosdep_mock, self.catkin_mock)
 
-        self.assertEqual(raised.args[0],
-                         "Package 'bar' isn't a valid system dependency. Did "
-                         "you forget to add it to catkin-packages? If not, "
-                         "add the Ubuntu package containing it to "
-                         "stage-packages until you can get it into the rosdep "
-                         "database.")
+        self.assertThat(str(raised), Equals(
+            "Package 'bar' isn't a valid system dependency. Did "
+            "you forget to add it to catkin-packages? If not, "
+            "add the Ubuntu package containing it to "
+            "stage-packages until you can get it into the rosdep "
+            "database."))
+
+    def test_find_system_dependencies_raises_if_unsupported_type(self):
+        self.rosdep_mock.resolve_dependency.return_value = {
+            'pip': {'baz'},
+        }
+
+        raised = self.assertRaises(
+            catkin.CatkinUnsupportedDependencyTypeError,
+            catkin._find_system_dependencies, {'foo'}, self.rosdep_mock,
+            self.catkin_mock)
+
+        self.assertThat(str(raised), Equals(
+            "Package 'bar' resolved to an unsupported type of dependency: "
+            "'pip'"))
 
 
 class HandleRosinstallFilesTestCase(tests.TestCase):
@@ -1150,157 +1283,6 @@ class HandleRosinstallFilesTestCase(tests.TestCase):
         ])
 
 
-class RosdepTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.project = snapcraft.ProjectOptions()
-        self.rosdep = catkin._Rosdep('kinetic', 'package_path',
-                                     'rosdep_path', 'sources',
-                                     self.project)
-
-        patcher = mock.patch('snapcraft.repo.Ubuntu')
-        self.ubuntu_mock = patcher.start()
-        self.addCleanup(patcher.stop)
-
-        patcher = mock.patch('subprocess.check_output')
-        self.check_output_mock = patcher.start()
-        self.addCleanup(patcher.stop)
-
-    def test_setup(self):
-        # Return something other than a Mock to ease later assertions
-        self.check_output_mock.return_value = b''
-
-        self.rosdep.setup()
-
-        # Verify that only rosdep was installed (no other .debs)
-        self.assertEqual(self.ubuntu_mock.call_count, 1)
-        self.assertEqual(self.ubuntu_mock.return_value.get.call_count, 1)
-        self.assertEqual(self.ubuntu_mock.return_value.unpack.call_count, 1)
-        self.ubuntu_mock.assert_has_calls([
-            mock.call(self.rosdep._rosdep_path, sources='sources',
-                      project_options=self.project),
-            mock.call().get(['python-rosdep']),
-            mock.call().unpack(self.rosdep._rosdep_install_path)])
-
-        # Verify that rosdep was initialized and updated
-        self.assertEqual(self.check_output_mock.call_count, 2)
-        self.check_output_mock.assert_has_calls([
-            mock.call(['rosdep', 'init'], env=mock.ANY),
-            mock.call(['rosdep', 'update'], env=mock.ANY)
-        ])
-
-    def test_setup_can_run_multiple_times(self):
-        self.rosdep.setup()
-
-        # Make sure running setup() again doesn't have problems with the old
-        # environment
-        # An exception will be raised if setup can't be called twice.
-        self.rosdep.setup()
-
-    def test_setup_initialization_failure(self):
-        def run(args, **kwargs):
-            if args == ['rosdep', 'init']:
-                raise subprocess.CalledProcessError(1, 'foo', b'bar')
-
-        self.check_output_mock.side_effect = run
-
-        raised = self.assertRaises(RuntimeError, self.rosdep.setup)
-
-        self.assertEqual(str(raised),
-                         'Error initializing rosdep database:\nbar')
-
-    def test_setup_update_failure(self):
-        def run(args, **kwargs):
-            if args == ['rosdep', 'update']:
-                raise subprocess.CalledProcessError(1, 'foo', b'bar')
-
-            return mock.DEFAULT
-
-        self.check_output_mock.side_effect = run
-
-        raised = self.assertRaises(RuntimeError, self.rosdep.setup)
-
-        self.assertEqual(str(raised),
-                         'Error updating rosdep database:\nbar')
-
-    def test_get_dependencies(self):
-        self.check_output_mock.return_value = b'foo\nbar\nbaz'
-
-        self.assertEqual(self.rosdep.get_dependencies('foo'),
-                         ['foo', 'bar', 'baz'])
-
-        self.check_output_mock.assert_called_with(['rosdep', 'keys', 'foo'],
-                                                  env=mock.ANY)
-
-    def test_get_dependencies_no_dependencies(self):
-        self.check_output_mock.return_value = b''
-
-        self.assertEqual(self.rosdep.get_dependencies('foo'), [])
-
-    def test_get_dependencies_invalid_package(self):
-        self.check_output_mock.side_effect = subprocess.CalledProcessError(
-            1, 'foo')
-
-        raised = self.assertRaises(
-            FileNotFoundError,
-            self.rosdep.get_dependencies, 'bar')
-
-        self.assertEqual(str(raised),
-                         'Unable to find Catkin package "bar"')
-
-    def test_resolve_dependency(self):
-        self.check_output_mock.return_value = b'#apt\nmylib-dev'
-
-        self.assertEqual(self.rosdep.resolve_dependency('foo'), ['mylib-dev'])
-
-        self.check_output_mock.assert_called_with(
-            ['rosdep', 'resolve', 'foo', '--rosdistro', 'kinetic', '--os',
-             'ubuntu:xenial'],
-            env=mock.ANY)
-
-    def test_resolve_invalid_dependency(self):
-        self.check_output_mock.side_effect = subprocess.CalledProcessError(
-            1, 'foo')
-
-        raised = self.assertRaises(
-            catkin.SystemDependencyNotFoundError,
-            self.rosdep.resolve_dependency, 'bar')
-
-        self.assertEqual(str(raised),
-                         "'bar' does not resolve to a system dependency")
-
-    def test_resolve_no_dependency(self):
-        self.check_output_mock.return_value = b'#apt'
-
-        self.assertEqual(self.rosdep.resolve_dependency('bar'), [])
-
-    def test_resolve_multiple_dependencies(self):
-        self.check_output_mock.return_value = b'#apt\nlib1 lib2'
-
-        self.assertEqual(self.rosdep.resolve_dependency('foo'),
-                         ['lib1', 'lib2'])
-
-    def test_run(self):
-        rosdep = self.rosdep
-        rosdep._run(['qux'])
-
-        class check_env():
-            def __eq__(self, env):
-                rosdep_sources_path = rosdep._rosdep_sources_path
-                return (
-                    env['PATH'] == os.path.join(rosdep._rosdep_install_path,
-                                                'usr', 'bin') and
-                    env['PYTHONPATH'] == os.path.join(
-                        rosdep._rosdep_install_path, 'usr', 'lib', 'python2.7',
-                        'dist-packages') and
-                    env['ROSDEP_SOURCE_PATH'] == rosdep_sources_path and
-                    env['ROS_HOME'] == rosdep._rosdep_cache_path and
-                    env['ROS_PACKAGE_PATH'] == rosdep._ros_package_path)
-
-        self.check_output_mock.assert_called_with(mock.ANY, env=check_env())
-
-
 class CompilersTestCase(tests.TestCase):
 
     def setUp(self):
@@ -1332,9 +1314,11 @@ class CompilersTestCase(tests.TestCase):
         self.compilers.setup()
 
         # Verify that both gcc and g++ were installed (no other .debs)
-        self.assertEqual(self.ubuntu_mock.call_count, 1)
-        self.assertEqual(self.ubuntu_mock.return_value.get.call_count, 1)
-        self.assertEqual(self.ubuntu_mock.return_value.unpack.call_count, 1)
+        self.assertThat(self.ubuntu_mock.call_count, Equals(1))
+        self.assertThat(
+            self.ubuntu_mock.return_value.get.call_count, Equals(1))
+        self.assertThat(
+            self.ubuntu_mock.return_value.unpack.call_count, Equals(1))
         self.ubuntu_mock.assert_has_calls([
             mock.call(self.compilers._compilers_path, sources='sources',
                       project_options=self.project),
@@ -1502,9 +1486,11 @@ class WstoolTestCase(tests.TestCase):
         self.wstool.setup()
 
         # Verify that only wstool was installed (no other .debs)
-        self.assertEqual(self.ubuntu_mock.call_count, 1)
-        self.assertEqual(self.ubuntu_mock.return_value.get.call_count, 1)
-        self.assertEqual(self.ubuntu_mock.return_value.unpack.call_count, 1)
+        self.assertThat(self.ubuntu_mock.call_count, Equals(1))
+        self.assertThat(
+            self.ubuntu_mock.return_value.get.call_count, Equals(1))
+        self.assertThat(
+            self.ubuntu_mock.return_value.unpack.call_count, Equals(1))
         self.ubuntu_mock.assert_has_calls([
             mock.call(self.wstool._wstool_path, sources='sources',
                       project_options=self.project),
