@@ -18,8 +18,9 @@ import os
 import shutil
 from unittest import mock
 
-from snapcraft.internal import sources
+from testtools.matchers import Equals
 
+from snapcraft.internal import sources
 from snapcraft import tests
 from snapcraft.tests.subprocess_utils import call
 
@@ -77,7 +78,7 @@ class TestSubversion(tests.sources.SourceTestCase):
             'svn://mysource', 'source_dir', source_tag='tag')
         expected_message = (
             "Can't specify source-tag for a Subversion source")
-        self.assertEqual(raised.message, expected_message)
+        self.assertThat(raised.message, Equals(expected_message))
 
     def test_init_with_source_branch_raises_exception(self):
         raised = self.assertRaises(
@@ -86,7 +87,7 @@ class TestSubversion(tests.sources.SourceTestCase):
             'svn://mysource', 'source_dir', source_branch='branch')
         expected_message = (
             "Can't specify source-branch for a Subversion source")
-        self.assertEqual(raised.message, expected_message)
+        self.assertThat(raised.message, Equals(expected_message))
 
     def test_init_with_source_branch_and_tag_raises_exception(self):
         raised = self.assertRaises(
@@ -98,7 +99,7 @@ class TestSubversion(tests.sources.SourceTestCase):
         expected_message = (
             "Can't specify source-tag OR source-branch for a Subversion "
             "source")
-        self.assertEqual(raised.message, expected_message)
+        self.assertThat(raised.message, Equals(expected_message))
 
     def test_init_with_source_depth_raises_exception(self):
         raised = self.assertRaises(
@@ -108,7 +109,7 @@ class TestSubversion(tests.sources.SourceTestCase):
 
         expected_message = (
             'can\'t specify source-depth for a Subversion source')
-        self.assertEqual(raised.message, expected_message)
+        self.assertThat(raised.message, Equals(expected_message))
 
     def test_source_checksum_raises_exception(self):
         raised = self.assertRaises(
@@ -119,7 +120,11 @@ class TestSubversion(tests.sources.SourceTestCase):
 
         expected_message = (
             "can't specify a source-checksum for a Subversion source")
-        self.assertEqual(raised.message, expected_message)
+        self.assertThat(raised.message, Equals(expected_message))
+
+    def test_has_source_handler_entry(self):
+        self.assertTrue(sources._source_handler['subversion'] is
+                        sources.Subversion)
 
 
 class SubversionBaseTestCase(tests.TestCase):
@@ -170,5 +175,5 @@ class SubversionDetailsTestCase(SubversionBaseTestCase):
         self.source_details = self.svn._get_source_details()
 
     def test_svn_details_commit(self):
-        self.assertEqual(
-            self.expected_commit, self.source_details['source-commit'])
+        self.assertThat(
+            self.source_details['source-commit'], Equals(self.expected_commit))
