@@ -22,7 +22,7 @@ from unittest import mock
 
 import fixtures
 import pymacaroons
-from testtools.matchers import Contains
+from testtools.matchers import Contains, Equals
 
 from snapcraft import (
     config,
@@ -143,10 +143,10 @@ class DownloadTestCase(StoreTestCase):
             errors.SnapNotFoundError,
             self.client.download,
             'unexisting-snap', 'test-channel', 'dummy', 'test-arch')
-        self.assertEqual(
-            "Snap 'unexisting-snap' for 'test-arch' cannot be found in "
-            "the 'test-channel' channel.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals("Snap 'unexisting-snap' for 'test-arch' cannot be found in "
+                   "the 'test-channel' channel."))
 
     def test_download_snap(self):
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
@@ -166,10 +166,11 @@ class DownloadTestCase(StoreTestCase):
             'test-snap-branded-store', 'test-channel', 'dummy')
 
         arch = ProjectOptions().deb_arch
-        self.assertEqual(
-            "Snap 'test-snap-branded-store' for '{}' cannot be found in "
-            "the 'test-channel' channel.".format(arch),
-            str(err))
+        self.assertThat(
+            str(err),
+            Equals(
+                "Snap 'test-snap-branded-store' for '{}' cannot be found in "
+                "the 'test-channel' channel.".format(arch)))
 
     def test_download_from_branded_store_requires_store(self):
         self.client.login('dummy', 'test correct password')
@@ -179,10 +180,11 @@ class DownloadTestCase(StoreTestCase):
             'test-snap-branded-store', 'test-channel', 'dummy')
 
         arch = ProjectOptions().deb_arch
-        self.assertEqual(
-            "Snap 'test-snap-branded-store' for '{}' cannot be found in "
-            "the 'test-channel' channel.".format(arch),
-            str(err))
+        self.assertThat(
+            str(err),
+            Equals(
+                "Snap 'test-snap-branded-store' for '{}' cannot be found in "
+                "the 'test-channel' channel.".format(arch)))
 
     def test_download_from_branded_store(self):
         # Downloading from a branded-store requires login (authorization)
@@ -261,19 +263,20 @@ class PushSnapBuildTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreSnapBuildError,
             self.client.push_snap_build, 'snap-id', 'test-not-implemented')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Could not assert build: The snap-build assertions are '
-            'currently disabled.')
+            Equals('Could not assert build: The snap-build assertions are '
+                   'currently disabled.'))
 
     def test_push_snap_build_invalid_data(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreSnapBuildError,
             self.client.push_snap_build, 'snap-id', 'test-invalid-data')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Could not assert build: The snap-build assertion is not valid.')
+            Equals('Could not assert build: The snap-build assertion is not '
+                   'valid.'))
 
     def test_push_snap_build_unexpected_data(self):
         # The endpoint in SCA would never return plain/text, however anything
@@ -282,9 +285,9 @@ class PushSnapBuildTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreSnapBuildError,
             self.client.push_snap_build, 'snap-id', 'test-unexpected-data')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Could not assert build: 500 Internal Server Error')
+            Equals('Could not assert build: 500 Internal Server Error'))
 
     def test_push_snap_build_successfully(self):
         self.client.login('dummy', 'test correct password')
@@ -301,110 +304,114 @@ class GetAccountInformationTestCase(StoreTestCase):
 
     def test_get_account_information_successfully(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual({
-            'account_id': 'abcd',
-            'account_keys': [],
-            'snaps': {
-                '16': {
-                    'basic': {
-                        'snap-id': 'snap-id',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'ubuntu-core': {
-                        'snap-id': 'good',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'core-no-dev': {
-                        'snap-id': 'no-dev',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'badrequest': {
-                        'snap-id': 'badrequest',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'test-snap-with-dev': {
-                        'price': None,
-                        'private': False,
-                        'since': '2016-12-12T01:01:01Z',
-                        'snap-id': 'test-snap-id-with-dev',
-                        'status': 'Approved'
-                    },
-                    'test-snap-with-no-validations': {
-                        'price': None,
-                        'private': False,
-                        'since': '2016-12-12T01:01:01Z',
-                        'snap-id': 'test-snap-id-with-no-validations',
-                        'status': 'Approved'
-                    },
+        self.assertThat(
+            self.client.get_account_information(),
+            Equals({
+                'account_id': 'abcd',
+                'account_keys': [],
+                'snaps': {
+                    '16': {
+                        'basic': {
+                            'snap-id': 'snap-id',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'ubuntu-core': {
+                            'snap-id': 'good',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'core-no-dev': {
+                            'snap-id': 'no-dev',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'badrequest': {
+                            'snap-id': 'badrequest',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'test-snap-with-dev': {
+                            'price': None,
+                            'private': False,
+                            'since': '2016-12-12T01:01:01Z',
+                            'snap-id': 'test-snap-id-with-dev',
+                            'status': 'Approved'
+                        },
+                        'test-snap-with-no-validations': {
+                            'price': None,
+                            'private': False,
+                            'since': '2016-12-12T01:01:01Z',
+                            'snap-id': 'test-snap-id-with-no-validations',
+                            'status': 'Approved'
+                        },
+                    }
                 }
-            }
-        }, self.client.get_account_information())
+            }))
 
     def test_get_account_information_refreshes_macaroon(self):
         self.client.login('dummy', 'test correct password')
         self.fake_store.needs_refresh = True
-        self.assertEqual({
-            'account_id': 'abcd',
-            'account_keys': [],
-            'snaps': {
-                '16': {
-                    'basic': {
-                        'snap-id': 'snap-id',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'ubuntu-core': {
-                        'snap-id': 'good',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'core-no-dev': {
-                        'snap-id': 'no-dev',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'badrequest': {
-                        'snap-id': 'badrequest',
-                        'status': 'Approved',
-                        'private': False,
-                        'price': None,
-                        'since': '2016-12-12T01:01:01Z',
-                    },
-                    'test-snap-with-dev': {
-                        'price': None,
-                        'private': False,
-                        'since': '2016-12-12T01:01:01Z',
-                        'snap-id': 'test-snap-id-with-dev',
-                        'status': 'Approved'
-                    },
-                    'test-snap-with-no-validations': {
-                        'price': None,
-                        'private': False,
-                        'since': '2016-12-12T01:01:01Z',
-                        'snap-id': 'test-snap-id-with-no-validations',
-                        'status': 'Approved'
-                    },
+        self.assertThat(
+            self.client.get_account_information(),
+            Equals({
+                'account_id': 'abcd',
+                'account_keys': [],
+                'snaps': {
+                    '16': {
+                        'basic': {
+                            'snap-id': 'snap-id',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'ubuntu-core': {
+                            'snap-id': 'good',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'core-no-dev': {
+                            'snap-id': 'no-dev',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'badrequest': {
+                            'snap-id': 'badrequest',
+                            'status': 'Approved',
+                            'private': False,
+                            'price': None,
+                            'since': '2016-12-12T01:01:01Z',
+                        },
+                        'test-snap-with-dev': {
+                            'price': None,
+                            'private': False,
+                            'since': '2016-12-12T01:01:01Z',
+                            'snap-id': 'test-snap-id-with-dev',
+                            'status': 'Approved'
+                        },
+                        'test-snap-with-no-validations': {
+                            'price': None,
+                            'private': False,
+                            'since': '2016-12-12T01:01:01Z',
+                            'snap-id': 'test-snap-id-with-no-validations',
+                            'status': 'Approved'
+                        },
+                    }
                 }
-            }
-        }, self.client.get_account_information())
+            }))
         self.assertFalse(self.fake_store.needs_refresh)
 
 
@@ -439,19 +446,19 @@ class RegisterKeyTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreKeyRegistrationError,
             self.client.register_key, 'test-not-implemented')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Key registration failed: 501 Not Implemented')
+            Equals('Key registration failed: 501 Not Implemented'))
 
     def test_invalid_data(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreKeyRegistrationError,
             self.client.register_key, 'test-invalid-data')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Key registration failed: '
-            'The account-key-request assertion is not valid.')
+            Equals('Key registration failed: '
+                   'The account-key-request assertion is not valid.'))
 
 
 class RegisterTestCase(StoreTestCase):
@@ -482,46 +489,47 @@ class RegisterTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreRegistrationError,
             self.client.register, 'test-already-registered-snap-name')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "The name 'test-already-registered-snap-name' is already taken."
-            "\n\n"
-            "We can if needed rename snaps to ensure they match the "
-            "expectations of most users. If you are the publisher most users "
-            "expect for 'test-already-registered-snap-name' then claim the "
-            "name at 'https://myapps.com/register-name/'")
+            Equals(
+                "The name 'test-already-registered-snap-name' is already "
+                "taken.\n\n"
+                "We can if needed rename snaps to ensure they match the "
+                "expectations of most users. If you are the publisher most "
+                "users expect for 'test-already-registered-snap-name' then "
+                "claim the name at 'https://myapps.com/register-name/'"))
 
     def test_register_a_reserved_name(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreRegistrationError,
             self.client.register, 'test-reserved-snap-name')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "The name 'test-reserved-snap-name' is reserved."
-            "\n\n"
-            "If you are the publisher most users expect for "
-            "'test-reserved-snap-name' then please claim the "
-            "name at 'https://myapps.com/register-name/'")
+            Equals("The name 'test-reserved-snap-name' is reserved."
+                   "\n\n"
+                   "If you are the publisher most users expect for "
+                   "'test-reserved-snap-name' then please claim the "
+                   "name at 'https://myapps.com/register-name/'"))
 
     def test_register_already_owned_name(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreRegistrationError,
             self.client.register, 'test-already-owned-snap-name')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "You already own the name 'test-already-owned-snap-name'.")
+            Equals("You already own the name 'test-already-owned-snap-name'."))
 
     def test_registering_too_fast_in_a_row(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreRegistrationError,
             self.client.register, 'test-too-fast')
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'You must wait 177 seconds before trying to register your '
-            'next snap.')
+            Equals('You must wait 177 seconds before trying to register your '
+                   'next snap.'))
 
     def test_registering_name_too_long(self):
         self.client.login('dummy', 'test correct password')
@@ -532,7 +540,7 @@ class RegisterTestCase(StoreTestCase):
         expected = (
             'The name {} should not be longer than 40 characters.'
             .format(name))
-        self.assertEqual(str(raised), expected)
+        self.assertThat(str(raised), Equals(expected))
 
     def test_registering_name_invalid(self):
         self.client.login('dummy', 'test correct password')
@@ -543,14 +551,14 @@ class RegisterTestCase(StoreTestCase):
         expected = (
             'The name {!r} is not valid. It can only contain dashes, numbers '
             'and lowercase ascii letters.'.format(name))
-        self.assertEqual(str(raised), expected)
+        self.assertThat(str(raised), Equals(expected))
 
     def test_unhandled_registration_error_path(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreRegistrationError,
             self.client.register, 'snap-name-no-clear-error')
-        self.assertEqual(str(raised), 'Registration failed.')
+        self.assertThat(str(raised), Equals('Registration failed.'))
 
 
 class ValidationsTestCase(StoreTestCase):
@@ -600,7 +608,7 @@ class ValidationsTestCase(StoreTestCase):
             "required": True,
         }]
         result = self.client.get_assertion('good', 'validations')
-        self.assertEqual(result, expected)
+        self.assertThat(result, Equals(expected))
 
     def test_get_bad_response(self):
         self.client.login('dummy', 'test correct password')
@@ -610,7 +618,7 @@ class ValidationsTestCase(StoreTestCase):
             self.client.get_assertion, 'bad', 'validations')
 
         expected = ("Received error 200: 'Invalid response from the server'")
-        self.assertEqual(str(err), expected)
+        self.assertThat(str(err), Equals(expected))
         self.assertIn(
             'Invalid response from the server', self.fake_logger.output)
 
@@ -631,7 +639,7 @@ class ValidationsTestCase(StoreTestCase):
         result = self.client.push_assertion('good', assertion, 'validations')
 
         expected = {'assertion': '{"foo": "bar"}'}
-        self.assertEqual(result, expected)
+        self.assertThat(result, Equals(expected))
 
     def test_push_bad_response(self):
         self.client.login('dummy', 'test correct password')
@@ -642,7 +650,7 @@ class ValidationsTestCase(StoreTestCase):
             self.client.push_assertion, 'bad', assertion, 'validations')
 
         expected = ("Received error 200: 'Invalid response from the server'")
-        self.assertEqual(str(err), expected)
+        self.assertThat(str(err), Equals(expected))
         self.assertIn(
             'Invalid response from the server', self.fake_logger.output)
 
@@ -655,7 +663,7 @@ class ValidationsTestCase(StoreTestCase):
             self.client.push_assertion, 'err', assertion, 'validations')
 
         expected = ("Received error 501: 'error'")
-        self.assertEqual(str(err), expected)
+        self.assertThat(str(err), Equals(expected))
 
 
 class UploadTestCase(StoreTestCase):
@@ -692,7 +700,7 @@ class UploadTestCase(StoreTestCase):
             'can_release': True,
             'processed': True
         }
-        self.assertEqual(result, expected_result)
+        self.assertThat(result, Equals(expected_result))
 
         # This should not raise
         tracker.raise_for_code()
@@ -709,7 +717,7 @@ class UploadTestCase(StoreTestCase):
             'can_release': True,
             'processed': True
         }
-        self.assertEqual(result, expected_result)
+        self.assertThat(result, Equals(expected_result))
 
         # This should not raise
         tracker.raise_for_code()
@@ -726,11 +734,11 @@ class UploadTestCase(StoreTestCase):
             errors.StoreUploadError,
             self.client.upload, 'test-snap', self.snap_path)
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'There was an error uploading the package.\n'
-            'Reason: \'Internal Server Error\'\n'
-            'Text: \'Broken\'')
+            Equals('There was an error uploading the package.\n'
+                   'Reason: \'Internal Server Error\'\n'
+                   'Text: \'Broken\''))
 
     def test_upload_snap_requires_review(self):
         self.client.login('dummy', 'test correct password')
@@ -744,7 +752,7 @@ class UploadTestCase(StoreTestCase):
             'can_release': False,
             'processed': True
         }
-        self.assertEqual(result, expected_result)
+        self.assertThat(result, Equals(expected_result))
 
         self.assertRaises(
             errors.StoreReviewError,
@@ -765,26 +773,28 @@ class UploadTestCase(StoreTestCase):
                 {'message': 'Duplicate snap already uploaded'},
             ]
         }
-        self.assertEqual(expected_result, result)
+        self.assertThat(result, Equals(expected_result))
 
         raised = self.assertRaises(
             errors.StoreReviewError,
             tracker.raise_for_code)
 
-        self.assertEqual(
-            'The store was unable to accept this snap.\n'
-            '  - Duplicate snap already uploaded', str(raised))
+        self.assertThat(
+            str(raised),
+            Equals('The store was unable to accept this snap.\n'
+                   '  - Duplicate snap already uploaded'))
 
     def test_push_unregistered_snap(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StorePushError,
             self.client.upload, 'test-snap-unregistered', self.snap_path)
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'You are not the publisher or allowed to push revisions for this '
-            'snap. To become the publisher, run `snapcraft register '
-            'test-snap-unregistered` and try to push again.')
+            Equals(
+                'You are not the publisher or allowed to push revisions for '
+                'this snap. To become the publisher, run `snapcraft register '
+                'test-snap-unregistered` and try to push again.'))
 
     def test_upload_with_invalid_credentials_raises_exception(self):
         conf = config.Config()
@@ -815,7 +825,7 @@ class ReleaseTestCase(StoreTestCase):
                 {'channel': 'edge', 'info': 'tracking'}
             ]
         }
-        self.assertEqual(channel_map, expected_channel_map)
+        self.assertThat(channel_map, Equals(expected_channel_map))
 
     def test_release_refreshes_macaroon(self):
         self.client.login('dummy', 'test correct password')
@@ -831,7 +841,7 @@ class ReleaseTestCase(StoreTestCase):
                 {'channel': 'edge', 'info': 'tracking'}
             ]
         }
-        self.assertEqual(channel_map, expected_channel_map)
+        self.assertThat(channel_map, Equals(expected_channel_map))
         self.assertFalse(self.fake_store.needs_refresh)
 
     def test_release_snap_to_invalid_channel(self):
@@ -840,9 +850,9 @@ class ReleaseTestCase(StoreTestCase):
             errors.StoreReleaseError,
             self.client.release, 'test-snap', '19', ['alpha'])
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Not a valid channel: alpha')
+            Equals('Not a valid channel: alpha'))
 
     def test_release_unregistered_snap(self):
         self.client.login('dummy', 'test correct password')
@@ -850,11 +860,11 @@ class ReleaseTestCase(StoreTestCase):
             errors.StoreReleaseError,
             self.client.release, 'test-snap-unregistered', '19', ['alpha'])
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Sorry, try `snapcraft register test-snap-unregistered` '
-            'before trying to release or choose an existing '
-            'revision.')
+            Equals('Sorry, try `snapcraft register test-snap-unregistered` '
+                   'before trying to release or choose an existing '
+                   'revision.'))
 
     def test_release_with_invalid_credentials_raises_exception(self):
         conf = config.Config()
@@ -888,10 +898,10 @@ class CloseChannelsTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreChannelClosingError,
             self.client.close_channels, 'snap-id', ['invalid'])
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "Could not close channel: The 'channels' field content "
-            "is not valid.")
+            Equals("Could not close channel: The 'channels' field content "
+                   "is not valid."))
 
     def test_close_unexpected_data(self):
         # The endpoint in SCA would never return plain/text, however anything
@@ -900,9 +910,9 @@ class CloseChannelsTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreChannelClosingError,
             self.client.close_channels, 'snap-id', ['unexpected'])
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Could not close channel: 500 Internal Server Error')
+            Equals('Could not close channel: 500 Internal Server Error'))
 
     def test_close_broken_store_plain(self):
         # If the contract is broken by the Store, users will be have additional
@@ -911,28 +921,32 @@ class CloseChannelsTestCase(StoreTestCase):
         raised = self.assertRaises(
             errors.StoreChannelClosingError,
             self.client.close_channels, 'snap-id', ['broken-plain'])
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Could not close channel: 200 OK')
-        self.assertEqual([
-            'Invalid response from the server on channel closing:',
-            '200 OK',
-            'b\'plain data\'',
-            ], self.fake_logger.output.splitlines()[-3:])
+            Equals('Could not close channel: 200 OK'))
+        self.assertThat(
+            self.fake_logger.output.splitlines()[-3:],
+            Equals([
+                'Invalid response from the server on channel closing:',
+                '200 OK',
+                'b\'plain data\'',
+            ]))
 
     def test_close_broken_store_json(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
             errors.StoreChannelClosingError,
             self.client.close_channels, 'snap-id', ['broken-json'])
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'Could not close channel: 200 OK')
-        self.assertEqual([
-            'Invalid response from the server on channel closing:',
-            '200 OK',
-            'b\'{"closed_channels": ["broken-json"]}\'',
-            ], self.fake_logger.output.splitlines()[-3:])
+            Equals('Could not close channel: 200 OK'))
+        self.assertThat(
+            self.fake_logger.output.splitlines()[-3:],
+            Equals([
+                'Invalid response from the server on channel closing:',
+                '200 OK',
+                'b\'{"closed_channels": ["broken-json"]}\'',
+            ]))
 
     def test_close_successfully(self):
         # Successfully closing a channels returns 'closed_channels'
@@ -940,19 +954,21 @@ class CloseChannelsTestCase(StoreTestCase):
         self.client.login('dummy', 'test correct password')
         closed_channels, channel_map_tree = self.client.close_channels(
             'snap-id', ['beta'])
-        self.assertEqual(['beta'], closed_channels)
-        self.assertEqual({
-            'latest': {
-                '16': {
-                    'amd64': [
-                        {'channel': 'stable', 'info': 'none'},
-                        {'channel': 'candidate', 'info': 'none'},
-                        {'channel': 'beta', 'info': 'specific',
-                         'revision': 42, 'version': '1.1'},
-                        {'channel': 'edge', 'info': 'tracking'}]
+        self.assertThat(closed_channels, Equals(['beta']))
+        self.assertThat(
+            channel_map_tree,
+            Equals({
+                'latest': {
+                    '16': {
+                        'amd64': [
+                            {'channel': 'stable', 'info': 'none'},
+                            {'channel': 'candidate', 'info': 'none'},
+                            {'channel': 'beta', 'info': 'specific',
+                             'revision': 42, 'version': '1.1'},
+                            {'channel': 'edge', 'info': 'tracking'}]
+                    }
                 }
-            }
-        }, channel_map_tree)
+            }))
 
 
 class MacaroonsTestCase(tests.TestCase):
@@ -1004,52 +1020,54 @@ class GetSnapRevisionsTestCase(StoreTestCase):
 
     def test_get_snap_revisions_successfully(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual(self.expected,
-                         self.client.get_snap_revisions('basic'))
+        self.assertThat(
+            self.client.get_snap_revisions('basic'),
+            Equals(self.expected))
 
     def test_get_snap_revisions_filter_by_series(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual(
-            self.expected,
-            self.client.get_snap_revisions('basic', series='16'))
+        self.assertThat(
+            self.client.get_snap_revisions('basic', series='16'),
+            Equals(self.expected))
 
     def test_get_snap_revisions_filter_by_arch(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual(
-            [rev for rev in self.expected if rev['arch'] == 'amd64'],
-            self.client.get_snap_revisions('basic', arch='amd64'))
+        self.assertThat(
+            self.client.get_snap_revisions('basic', arch='amd64'),
+            Equals([rev for rev in self.expected if rev['arch'] == 'amd64']))
 
     def test_get_snap_revisions_filter_by_series_and_filter(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual(
-            [rev for rev in self.expected
-             if '16' in rev['series'] and rev['arch'] == 'amd64'],
+        self.assertThat(
             self.client.get_snap_revisions(
-                'basic', series='16', arch='amd64'))
+                'basic', series='16', arch='amd64'),
+            Equals([rev for rev in self.expected
+                    if '16' in rev['series'] and rev['arch'] == 'amd64']))
 
     def test_get_snap_revisions_filter_by_unknown_series(self):
         self.client.login('dummy', 'test correct password')
         e = self.assertRaises(
             storeapi.errors.SnapNotFoundError,
             self.client.get_snap_revisions, 'basic', series='12')
-        self.assertEqual(
-            "Snap 'basic' was not found in '12' series.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals("Snap 'basic' was not found in '12' series."))
 
     def test_get_snap_revisions_filter_by_unknown_arch(self):
         self.client.login('dummy', 'test correct password')
         e = self.assertRaises(
             storeapi.errors.SnapNotFoundError,
             self.client.get_snap_revisions, 'basic', arch='somearch')
-        self.assertEqual(
-            "Snap 'basic' for 'somearch' was not found in '16' series.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals(
+                "Snap 'basic' for 'somearch' was not found in '16' series."))
 
     def test_get_snap_revisions_refreshes_macaroon(self):
         self.client.login('dummy', 'test correct password')
         self.fake_store.needs_refresh = True
-        self.assertEqual(self.expected,
-                         self.client.get_snap_revisions('basic'))
+        self.assertThat(
+            self.client.get_snap_revisions('basic'), Equals(self.expected))
         self.assertFalse(self.fake_store.needs_refresh)
 
     @mock.patch.object(storeapi.StoreClient, 'get_account_information')
@@ -1069,10 +1087,11 @@ class GetSnapRevisionsTestCase(StoreTestCase):
         e = self.assertRaises(
             storeapi.errors.StoreSnapRevisionsError,
             self.client.get_snap_revisions, 'basic')
-        self.assertEqual(
-            "Error fetching revisions of snap id 'my_snap_id' for 'any arch' "
-            "in '16' series: 500 Server error.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals(
+                "Error fetching revisions of snap id 'my_snap_id' for "
+                "'any arch' in '16' series: 500 Server error."))
 
 
 class GetSnapStatusTestCase(StoreTestCase):
@@ -1129,63 +1148,70 @@ class GetSnapStatusTestCase(StoreTestCase):
 
     def test_get_snap_status_successfully(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual(self.expected, self.client.get_snap_status('basic'))
+        self.assertThat(
+            self.client.get_snap_status('basic'),
+            Equals(self.expected))
 
     def test_get_snap_status_filter_by_series(self):
         self.client.login('dummy', 'test correct password')
-        self.assertEqual(
-            self.expected,
-            self.client.get_snap_status('basic', series='16'))
+        self.assertThat(
+            self.client.get_snap_status('basic', series='16'),
+            Equals(self.expected))
 
     def test_get_snap_status_filter_by_arch(self):
         self.client.login('dummy', 'test correct password')
         exp_arch = self.expected['channel_map_tree']['latest']['16']['amd64']
-        self.assertEqual(
-            {'channel_map_tree': {
-                'latest': {
-                    '16': {
-                        'amd64': exp_arch
+        self.assertThat(
+            self.client.get_snap_status('basic', arch='amd64'),
+            Equals(
+                {'channel_map_tree': {
+                    'latest': {
+                        '16': {
+                            'amd64': exp_arch
+                        }
                     }
-                }
-            }},
-            self.client.get_snap_status('basic', arch='amd64'))
+                }}))
 
     def test_get_snap_status_filter_by_series_and_filter(self):
         self.client.login('dummy', 'test correct password')
         exp_arch = self.expected['channel_map_tree']['latest']['16']['amd64']
-        self.assertEqual(
-            {'channel_map_tree': {
-                'latest': {
-                    '16': {
-                        'amd64': exp_arch
-                    }
-                }
-            }},
+        self.assertThat(
             self.client.get_snap_status(
-                'basic', series='16', arch='amd64'))
+                'basic', series='16', arch='amd64'),
+            Equals(
+                {'channel_map_tree': {
+                    'latest': {
+                        '16': {
+                            'amd64': exp_arch
+                        }
+                    }
+                }}))
 
     def test_get_snap_status_filter_by_unknown_series(self):
         self.client.login('dummy', 'test correct password')
         e = self.assertRaises(
             storeapi.errors.SnapNotFoundError,
             self.client.get_snap_status, 'basic', series='12')
-        self.assertEqual(
-            "Snap 'basic' was not found in '12' series.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals("Snap 'basic' was not found in '12' series."))
 
     def test_get_snap_status_filter_by_unknown_arch(self):
         self.client.login('dummy', 'test correct password')
         e = self.assertRaises(
             storeapi.errors.SnapNotFoundError,
             self.client.get_snap_status, 'basic', arch='somearch')
-        self.assertEqual(
-            "Snap 'basic' for 'somearch' was not found in '16' series.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals(
+                "Snap 'basic' for 'somearch' was not found in '16' series."))
 
     def test_get_snap_status_refreshes_macaroon(self):
         self.client.login('dummy', 'test correct password')
         self.fake_store.needs_refresh = True
-        self.assertEqual(self.expected, self.client.get_snap_status('basic'))
+        self.assertThat(
+            self.client.get_snap_status('basic'),
+            Equals(self.expected))
         self.assertFalse(self.fake_store.needs_refresh)
 
     @mock.patch.object(storeapi.StoreClient, 'get_account_information')
@@ -1202,10 +1228,11 @@ class GetSnapStatusTestCase(StoreTestCase):
         e = self.assertRaises(
             storeapi.errors.StoreSnapStatusError,
             self.client.get_snap_status, 'basic')
-        self.assertEqual(
-            "Error fetching status of snap id 'my_snap_id' for 'any arch' "
-            "in '16' series: 500 Server error.",
-            str(e))
+        self.assertThat(
+            str(e),
+            Equals(
+                "Error fetching status of snap id 'my_snap_id' for 'any arch' "
+                "in '16' series: 500 Server error."))
 
 
 class SignDeveloperAgreementTestCase(StoreTestCase):
@@ -1220,9 +1247,11 @@ class SignDeveloperAgreementTestCase(StoreTestCase):
                 "accepted_tos_date": "2010-10-10"
                 }
             }
-        self.assertEqual(
+        self.assertThat(
             response,
-            self.client.sign_developer_agreement(latest_tos_accepted=True))
+            Equals(
+                self.client.sign_developer_agreement(
+                    latest_tos_accepted=True)))
 
     def test_sign_dev_agreement_exception(self):
         self.client.login('dummy', 'test correct password')
@@ -1241,8 +1270,8 @@ class SignDeveloperAgreementTestCase(StoreTestCase):
             errors.DeveloperAgreementSignError,
             self.client.sign_developer_agreement,
             latest_tos_accepted=True)
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            'There was an error while signing developer agreement.\n'
-            'Reason: \'Internal Server Error\'\n'
-            'Text: \'Broken\'')
+            Equals('There was an error while signing developer agreement.\n'
+                   'Reason: \'Internal Server Error\'\n'
+                   'Text: \'Broken\''))
