@@ -16,7 +16,12 @@
 
 import os
 
-from snapcraft.internal import common
+from testtools.matchers import Equals
+
+from snapcraft.internal import (
+    common,
+    errors
+)
 from snapcraft import tests
 
 
@@ -25,12 +30,7 @@ class CommonTestCase(tests.TestCase):
     def test_set_plugindir(self):
         plugindir = os.path.join(self.path, 'testplugin')
         common.set_plugindir(plugindir)
-        self.assertEqual(plugindir, common.get_plugindir())
-
-    def test_set_tourdir(self):
-        tourdir = os.path.join(self.path, 'testtour')
-        common.set_tourdir(tourdir)
-        self.assertEqual(tourdir, common.get_tourdir())
+        self.assertThat(plugindir, Equals(common.get_plugindir()))
 
     def test_isurl(self):
         self.assertTrue(common.isurl('git://'))
@@ -44,31 +44,30 @@ class CommonMigratedTestCase(tests.TestCase):
 
     def test_parallel_build_count_migration_message(self):
         raised = self.assertRaises(
-            EnvironmentError,
+            errors.PluginOutdatedError,
             common.get_parallel_build_count)
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "This plugin is outdated, use "
-            "'parallel_build_count'")
+            Equals("This plugin is outdated: use 'parallel_build_count'"))
 
     def test_deb_arch_migration_message(self):
         raised = self.assertRaises(
-            EnvironmentError,
+            errors.PluginOutdatedError,
             common.get_arch)
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "This plugin is outdated, use 'project.deb_arch'")
+            Equals("This plugin is outdated: use 'project.deb_arch'"))
 
     def test_arch_triplet_migration_message(self):
         raised = self.assertRaises(
-            EnvironmentError,
+            errors.PluginOutdatedError,
             common.get_arch_triplet)
 
-        self.assertEqual(
+        self.assertThat(
             str(raised),
-            "This plugin is outdated, use 'project.arch_triplet'")
+            Equals("This plugin is outdated: use 'project.arch_triplet'"))
 
 
 class FormatInColumnsTestCase(tests.TestCase):
@@ -83,33 +82,37 @@ class FormatInColumnsTestCase(tests.TestCase):
                     'nodejs   python3  tar-content',
                     'autotools  cmake   go    kbuild  make    nil    '
                     'python2  scons  ']
-        self.assertEquals(expected,
-                          common.format_output_in_columns(self.elements_list))
+        self.assertThat(
+            common.format_output_in_columns(self.elements_list),
+            Equals(expected))
 
     def test_format_output_in_columns_narrow(self):
         """Format output on 3 lines, with narrow max-width and space sep"""
         expected = ['ant        cmake  jdk     make   nodejs   scons      ',
                     'autotools  copy   kbuild  maven  python2  tar-content',
                     'catkin     go     kernel  nil    python3']
-        self.assertEquals(expected,
-                          common.format_output_in_columns(self.elements_list,
-                                                          max_width=60))
+        self.assertThat(
+            common.format_output_in_columns(self.elements_list,
+                                            max_width=60),
+            Equals(expected))
 
     def test_format_output_in_columns_large(self):
         """Format output on one big line, with default space sep"""
         expected = ['ant  autotools  catkin  cmake  copy  go  jdk  kbuild  '
                     'kernel  make  maven  nil  nodejs  python2  python3  '
                     'scons  tar-content']
-        self.assertEquals(expected,
-                          common.format_output_in_columns(self.elements_list,
-                                                          max_width=990))
+        self.assertThat(
+            common.format_output_in_columns(self.elements_list,
+                                            max_width=990),
+            Equals(expected))
 
     def test_format_output_in_columns_one_space(self):
         """Format output with one space sep"""
         expected = ['ant       cmake jdk    make  nodejs  scons      ',
                     'autotools copy  kbuild maven python2 tar-content',
                     'catkin    go    kernel nil   python3']
-        self.assertEquals(expected,
-                          common.format_output_in_columns(self.elements_list,
-                                                          max_width=60,
-                                                          num_col_spaces=1))
+        self.assertThat(
+            common.format_output_in_columns(self.elements_list,
+                                            max_width=60,
+                                            num_col_spaces=1),
+            Equals(expected))
