@@ -17,10 +17,12 @@
 import os
 import re
 import subprocess
-from platform import linux_distribution
 from unittest import skipUnless
 
+from testtools.matchers import Equals
+
 import snapcraft
+from snapcraft.internal.common import get_os_release_info
 import snaps_tests
 
 
@@ -28,7 +30,7 @@ class ROSTestCase(snaps_tests.SnapsTestCase):
 
     snap_content_dir = 'ros'
 
-    @skipUnless(linux_distribution()[2] == 'xenial',
+    @skipUnless(get_os_release_info()['VERSION_CODENAME'] == 'xenial',
                 'This test fails on yakkety LP: #1614476')
     def test_ros(self):
         try:
@@ -51,7 +53,7 @@ class ROSTestCase(snaps_tests.SnapsTestCase):
         output = subprocess.check_output(
             "sed -n '/env/p;1q' prime/usr/bin/rosversion",
             cwd=os.path.join(self.path, self.snap_content_dir), shell=True)
-        self.assertEqual(output, expected)
+        self.assertThat(output, Equals(expected))
 
         # Regression test for LP: #1660852. Make sure --help actually gets
         # passed to rosaunch instead of being eaten by setup.sh.
